<?php
class ControllerPaymentFlow extends Controller {
	private $error = array();

	public function index() {
		
		$this->load->language('payment/flow');

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model('setting/setting');
		
		$privData = $this->getData();
		$privData['flow_privatekey'] = $this->config->get('flow_privatekey');
		
		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate($privData)) {
			$config = $this->request->post;
			if ($this->privateKeyData) {
				$config['flow_privatekey'] = $this->privateKeyData;
			} else {
				$config['flow_privatekey'] = $privData['flow_privatekey'];
			}
			
			$this->model_setting_setting->editSetting('flow', $config);

			$this->session->data['success'] = $this->language->get('text_success');
			
			$url = $this->url->link('extension/payment', 'token=' . $this->session->data['token'], 'SSL');
			if ($this->isVersion1()) {
				$this->redirect($url);
			} else {
				$this->response->redirect($url);
			}
		}

		$this->setTranslations($privData);
		$this->setErrorInfo($privData);
		$this->setBreadcrumbs($privData);
		$this->updateEditors($privData);
		
		if (isset($this->session->data['success'])) {
			$privData['success'] = $this->session->data['success'];
		
			unset($this->session->data['success']);
		} else {
			$privData['success'] = '';
		}
		
		$flow_skip_types = array();
		$flow_skip_types[] = array('id' =>'d', 'name'=>'Ingresar directamente a Webpay');
		$flow_skip_types[] = array('id' =>'f', 'name'=>'Mostrar pasarela Flow');
		$privData['flow_skip_types'] = $flow_skip_types;
		
		$privData['action'] = $this->url->link('payment/flow', 'token=' . $this->session->data['token'], 'SSL');

		$privData['cancel'] = $this->url->link('extension/payment', 'token=' . $this->session->data['token'], 'SSL');


		$this->load->model('localisation/order_status');
		$privData['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

		$this->load->model('localisation/geo_zone');
		$privData['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();
		
		if ($this->isVersion1()) {
			$this->data = $privData;
			$this->template = 'payment/flow.tpl';
			$this->children = array(
				'common/header',
				'common/footer'
			);
	
			$this->response->setOutput($this->render());
		} else {
			$privData['header'] = $this->load->controller('common/header');
			$privData['footer'] = $this->load->controller('common/footer');
			$privData['column_left'] = $this->load->controller('common/column_left');
			$this->response->setOutput($this->load->view('payment/flow2.tpl', $privData));
		}
	}
	
	
	private function updateEditors(&$privData) {
		$this->updateEditor($privData, 'flow_platform_id');
		$this->updateEditor($privData, 'flow_receiverid');
		$this->updateEditor($privData, 'flow_title');
		$this->updateEditor($privData, 'flow_completed_status_id');
		$this->updateEditor($privData, 'flow_failed_status_id');
		$this->updateEditor($privData, 'flow_geo_zone_id');
		$this->updateEditor($privData, 'flow_status');
		$this->updateEditor($privData, 'flow_sort_order');
		$this->updateEditor($privData, 'flow_skip_type_id');
	}
	
	private function setTranslations(&$privData) {
		$privData['heading_title'] = $this->language->get('heading_title');
		$privData['text_edit'] = $this->language->get('text_edit');
		
		$privData['text_all_zones'] = $this->language->get('text_all_zones');
		$privData['text_enabled'] = $this->language->get('text_enabled');
		$privData['text_disabled'] = $this->language->get('text_disabled');
		
		$privData['entry_geo_zone'] = $this->language->get('entry_geo_zone');
		$privData['entry_status'] = $this->language->get('entry_status');
		$privData['entry_sort_order'] = $this->language->get('entry_sort_order');
		
		$privData['entry_receiverid'] = $this->language->get('entry_receiverid');
		$privData['entry_title'] = $this->language->get('entry_title');
		$privData['entry_secret'] = $this->language->get('entry_secret');
		$privData['entry_completed_status'] = $this->language->get('entry_completed_status');
		$privData['entry_failed_status'] = $this->language->get('entry_failed_status');
		$privData['entry_type'] = $this->language->get('entry_type');
		$privData['entry_skip_type'] = $this->language->get('entry_skip_type');
		
		$privData['placeholder_receiverid'] = $this->language->get('placeholder_receiverid');
		$privData['placeholder_title'] = $this->language->get('placeholder_title');
		$privData['tip_status_complete'] = $this->language->get('tip_status_complete');
		$privData['tip_status_failed']   = $this->language->get('tip_status_failed');
		
		$privData['flow_type_default'] = $this->language->get('type_default');
		
		$privData['entry_platform'] = $this->language->get('entry_platform');
		$privData['flow_test_site'] = $this->language->get('flow_test_site');
		$privData['flow_site'] = $this->language->get('flow_site');
		
		$privData['button_save'] = $this->language->get('button_save');
		$privData['button_cancel'] = $this->language->get('button_cancel');
		
		$privData['has_private_key'] = $this->language->get('has_private_key');
	}
	
	private function setErrorInfo(&$privData) {
		$this->handleError($privData, 'warning');
		$this->handleError($privData, 'receiverid');
		$this->handleError($privData, 'title');
		$this->handleError($privData, 'privatekey');
		$this->handleError($privData, 'type');
	}
	
	private function setBreadcrumbs(&$privData) {
		$privData['breadcrumbs'] = array();
		$this->addBreadCrumb($privData, 'text_home', 'common/home', false);
		$this->addBreadCrumb($privData, 'text_payment', 'extension/payment', ' :: ');
		$this->addBreadCrumb($privData, 'heading_title', 'payment/flow', ' :: ');
	}

	private function isValidPrivateKey($privateKeyFile) {
		$fp = fopen($privateKeyFile, "r");
		$priv_key = fread($fp, 8192);
		fclose($fp);
		return openssl_get_privatekey($priv_key)!=null;
	}
	
	private function validate($privData) {
		if (!$this->user->hasPermission('modify', 'payment/flow')) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		if (!$this->request->post['flow_receiverid']) {
			$this->error['receiverid'] = $this->language->get('error_receiverid');
		}
        
		if (!$this->request->post['flow_title']) {
			$this->error['title'] = $this->language->get('error_title');
		}
		
		$hasFile = false;
        if (isset($_FILES['flow_privatekey_file'])) {
        	$file = $_FILES['flow_privatekey_file'];
        	$hasFile = $file['size']>0;
        	if ($hasFile) {
        		$fileName = $file['tmp_name'];
        		if (!$this->isValidPrivateKey($fileName)) {
        			$this->error['privatekey'] = $this->language->get('error_privatekey_invalid');
        		} else {
        			$this->privateKeyData = file_get_contents($fileName);
        		}
        	}
        } 
        
        if (!isset($privData['flow_privatekey']) && !$hasFile) {
        	$this->error['privatekey'] = $this->language->get('error_privatekey');
        }
        
		if (!$this->error) {
			return true;
		} else {
			return false;
		}
	}
	
	// Utility functions
	
	private function handleError(&$privData, $type) {
		if (isset($this->error[$type])) {
			$privData['error_'.$type] = $this->error[$type];
		} else {
			$privData['error_'.$type] = '';
		}
	}

	private function addBreadCrumb(&$privData, $title, $link, $separator) {
		$privData['breadcrumbs'][] = array(
				'text'      => $this->language->get($title),
				'href'      => $this->url->link($link, 'token=' . $this->session->data['token'], 'SSL'),
				'separator' => $separator
		);
	}

	private function updateEditor(&$privData, $editor) {
		if (isset($this->request->post[$editor])) {
			$privData[$editor] = $this->request->post[$editor];
		} else {
			$privData[$editor] = $this->config->get($editor);
		}
	}
	
	private function getData() {
		if ($this->isVersion1()) {
			return $this->data;
		}
		return array();
	}
	
	private function isVersion1() {
		return version_compare(VERSION, '2.0.0.0', '<');
	}
}
?>
